﻿using System;
using System.Collections.Generic;
using UnityEngine;

/// <summary>
/// 移動のやり直し機能
/// 
/// オブジェクト数が20を超えている場合は使用できません
/// </summary>
public class UndoV3 : MonoBehaviour
{
    [SerializeField, TooltipAttribute("記憶対象のPrize")] GameObject targetObjects;
    [SerializeField] bool player2 = false;
    private Transform targetParent;
    private CraneManagerV3 c;
    private MachineHost host;

    // private
    // 今のところ1手戻ることしか考えてないのでとりあえずList
    private List<MoveHistroy> historys = new List<MoveHistroy>();

    void Start()
    {
        c = this.GetComponent<CraneManagerV3>();
        host = c.host;
        if (c == null || targetObjects == null)
        {
            // なければエラー
            Debug.LogError("エラー: UndoV3");
        }
        if (targetObjects.activeSelf)
        {
            targetObjects.SetActive(false);
        }
        Invoke("Init", 1f);
    }

    void Init()
    {
        // 初期化
    }

    void Update()
    {
        if (host.playable)
        {
            if (!player2 && (Input.GetKeyDown(KeyCode.F5)))
            {
                StateSave();
            }
            if (!player2 && (Input.GetKeyDown(KeyCode.F6)))
            {
                Undo();
            }
        }
    }

    public void StateSave()
    {
        Debug.Log("位置を保存する");
        // 位置保存
        if (!player2)
        {
            this.FindPrizeObject("Prize1(Clone)", 0);
        }
        else
        {
            this.FindPrizeObject("Prize2(Close)", 0);
        }
    }

    public void Undo()
    {
        Debug.Log("位置を戻す");
        // 位置戻し
        if (!player2)
        {
            this.FindPrizeObject("Prize1(Clone)", 1);
        }
        else
        {
            this.FindPrizeObject("Prize2(Close)", 1);
        }
    }

    /// <summary>
    /// 景品オブジェクトを検索&なにかする
    /// </summary>
    /// <param name="_parentName">Prizeのルートフォルダ名称</param>
    /// <param name="_mode">0:座標保存 1:座標読み込み</param>
    private void FindPrizeObject(string _parentName, int _mode)
    {
        // とりあえずこれで1P側プライズの階層オブジェクトが取得できる
        // これの子オブジェクトを全部回してリストに入れる感じ
        var prize = transform.parent.Find(_parentName).gameObject;
        var childObject = prize.GetComponentInChildren<Transform>();

        if (childObject.childCount > 20)
        {
            Debug.LogError("オブジェクト数が多すぎるので使えません");
            return;
        }

        // TODO: 再帰的参照
        foreach (Transform obj in childObject)
        {
            // タグがprizeでないかつ子を持つ場合は再帰的参照
            if (obj.tag != "prize" && obj.childCount >= 1)
            {
                Debug.Log("子を持つプライズ以外のオブジェクトなので下の階層を調べる");
                this.GetMoreChildObject(obj, _mode);
                continue;
            }
            // タグがprizeでないかつ子を持たない場合は履歴に入れない
            else if (obj.tag != "prize" && obj.childCount == 0)
            {
                Debug.Log("子を持たないプライズ以外のオブジェクトなので次へ");
                continue;
            }

            // 景品オブジェクトへのアクション
            if (_mode == 0)
            {
                // リストに追加
                this.StateSet(obj, $"{obj.name}");
            }
            else if (_mode == 1)
            {
                // リストから読み込み
                this.StateLoad(obj, $"{obj.name}");
            }
            else
            {
                Debug.LogError($"不正なmode値が渡されました mode={_mode}");
            }
        }
    }

    private void GetMoreChildObject(Transform _obj, int _mode)
    {
        // 1層下のオブジェクトを取得する
        // 子を探す
        foreach (Transform obj in _obj)
        {
            if (obj.tag != "prize")
            {
                Debug.Log("プライズ以外のオブジェクトなので次へ");
            }
            Debug.Log($"オブジェクト名:{obj.parent.name}/{obj.name}");

            // 景品オブジェクトへのアクション
            if (_mode == 0)
            {
                // リストに追加
                this.StateSet(obj, $"{obj.parent.name}/{obj.name}");
            }
            else if (_mode == 1)
            {
                // リストから読み込み
                this.StateLoad(obj, $"{obj.parent.name}/{obj.name}");
            }
            else
            {
                Debug.LogError($"不正なmode値が渡されました mode={_mode}");
            }
        }
    }

    private void StateSet(Transform _obj, string _objectPath)
    {
        // リスト内に既にあったら消す
        var o = this.historys.Find(x => x.objectName == _objectPath);
        if (o is not null)
        {
            this.historys.Remove(o);
        }

        // 履歴から座標を設定する
        this.historys.Add(new MoveHistroy
        {
            objectName = _objectPath,
            localPosition = _obj.localPosition,
            eulerAngles = _obj.eulerAngles,
        });
    }

    private void StateLoad(Transform _obj, string _objectPath)
    {
        var o = this.historys.Find(x => x.objectName == _objectPath);

        if (o is not null)
        {
            _obj.localPosition = o.localPosition;
            _obj.eulerAngles = o.eulerAngles;
        }
        else
        {
            Debug.Log($"履歴に登録されていません path:{_objectPath}");
        }
    }

    public void ResetHistory()
    {
        // TODO: 実装
    }
}

public class MoveHistroy
{
    // TODO: オブジェクトの識別方法
    public string objectName { get; set; }
    public Vector3 localPosition { get; set; }
    public Vector3 eulerAngles { get; set; }
}
